!   Linux kernel setup for AMD lan SC4x0 based devices
!
!   Portions are Copyright (C) 2000 by Denis Hatebur and TueViT Essen
!   Portions are Copyright (C) 2000 by Anders Larsen and Baumer Ident GmbH
!
!   Derived from setup.S   Copyright (C) 1991-1997 Linus Torvalds et al.
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation; either version 2 of the License, or
!   (at your option) any later version.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!
!   You should have received a copy of the GNU General Public License
!   along with this program. If not, write to the Free Software
!   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	.org 0x0
	.text

USE32

entry CODE1
CODE1:
	cli			! Disable Interrupts

	mov al, #0x0a
	call ch_out

	mov al, #0x0d
	call ch_out

	mov al, #'i
	call ch_out

	mov al, #0x80
	out #0x70, al		! Disable NMIs

	mov al, #'-
	call ch_out

	mov al, #'P
	call ch_out

	mov al, #'O
	call ch_out

	mov al, #'R
	call ch_out

	mov al, #'T
	call ch_out

	mov al, #0x0a
	call ch_out

	mov al, #0x0d
	call ch_out

	mov eax, cr0
	mov al,  #0x01
	mov cr0, eax
	mov edi, #0x60000
	seg ds
	mov [edi], eax

	mov edi, kernel_version+0x90000
	call str_out

	mov ecx,  start+0x90000
	jmp ecx


	.org	0x200
start:
	jmp	start_of_setup
! ------------------------ start of header --------------------------------
!
! SETUP-header, must start at CS:2 (old 0x9020:2)
!
		.ascii	"HdrS"		! Signature for SETUP-header
		.word	0x0201		! Version number of header format
					! (must be >= 0x0105
					! else old loadlin-1.5 will fail)
realmode_swtch:	.word	0,0		! default_switch,SETUPSEG
start_sys_seg:	.word	0x1000
		.word	kernel_version	!pointing to kernel version string
  ! note: above part of header is compatible with loadlin-1.5 (header v1.5),
  !        must not change it

type_of_loader:	.byte	1		! = 0, old one (LILO, Loadlin,
					!      Bootlin, SYSLX, bootsect...)
					! else it is set by the loader:
					! 0xTV: T=0 for LILO
					!	T=1 for Loadlin
					!	T=2 for bootsect-loader
					!	T=3 for SYSLX
					!	T=4 for ETHERBOOT
					!       V = version
loadflags:				! flags, unused bits must be zero (RFU)
		.byte	0x81


setup_move_size: .word  0x8000		! size to move, when we (setup) are not
					! loaded at 0x90000. We will move ourselves
					! to 0x90000 then just before jumping into
					! the kernel. However, only the loader
					! know how much of data behind us also needs
					! to be loaded.
code32_start:				! here loaders can put a different
		.long	0x100000	! 0x100000 = default for big kernel


ramdisk_image:	.long	0		! address of loaded ramdisk image
					! Here the loader (or kernel generator) puts
					! the 32-bit address were it loaded the image.
					! This only will be interpreted by the kernel.
ramdisk_size:	.long	0		! its size in bytes
bootsect_kludge:
		.word   0,0
heap_end_ptr:	.word	0x9e00		! space from here (exclusive) down to
					! end of setup code can be used by setup
					! for local heap purposes.
mem_size:	.long	0x0003c00

! ------------------------ end of header ----------------------------------

start_of_setup:
	
	mov ax, #0xa33f
	mov [0x90020], ax		; magic number for kernel parameter

	mov ax, #kernelparam
	mov [0x90022], ax		; relative address (to 0x90000) for kernel parameter


	mov edi, #mem_size+0x90000
        mov eax, [edi]
	mov [0x901e0], eax		; MEMORY-SIZE NEW

	mov edi, #mem_size+0x90000
        mov eax, [edi]
	mov [0x90002], ax		; MEMORY-SIZE OLD

	mov ax, ds
	mov es, ax
	mov edi, #0x90080
	mov ecx, #0x00020
	xor ax, ax
	cld
	rep
	stosb                  		; NO HD1,  NO HD2
					; -> NO DISK
	mov ax, #0x0
	mov [0x900A0], ax		; No Micro Channel (MCA) Bus

	mov al, #0x0
	mov [0x901ff], al		; No PS/2 Mice

	mov al, #0x0
	mov [0x90040], al		; No APM BIOS

	mov al, #0x0
	mov [0x90064], al		; No APM BIOS

	seg cs
	lidt	[idt_48+0x90000]	! load idt with 0,0
	seg cs
	lgdt	[gdt_48+0x90000]	! load gdt with whatever appropriate

! make sure any possible coprocessor is properly reset..

	xor	ax,ax
	out	#0xf0,al
	call	delay
	out	#0xf1,al
	call	delay

! well, that went ok, I hope. Now we have to reprogram the interrupts :-(
! we put them right after the intel-reserved hardware interrupts, at
! int 0x20-0x2F. There they won't mess up anything. Sadly IBM really
! messed this up with the original PC, and they haven't been able to
! rectify it afterwards. Thus the bios puts interrupts at 0x08-0x0f,
! which is used for the internal hardware interrupts as well. We just
! have to reprogram the 8259's, and it isn't fun.

	mov	al,#0x11		! initialization sequence
	out	#0x20,al		! send it to 8259A-1
	call	delay
	out	#0xA0,al		! and to 8259A-2
	call	delay
	mov	al,#0x20		! start of hardware int's (0x20)
	out	#0x21,al
	call	delay
	mov	al,#0x28		! start of hardware int's 2 (0x28)
	out	#0xA1,al
	call	delay
	mov	al,#0x04		! 8259-1 is master
	out	#0x21,al
	call	delay
	mov	al,#0x02		! 8259-2 is slave
	out	#0xA1,al
	call	delay
	mov	al,#0x01		! 8086 mode for both
	out	#0x21,al
	call	delay
	out	#0xA1,al
	call	delay
	mov	al,#0xFF		! mask off all interrupts for now
	out	#0xA1,al
	call	delay
	mov	al,#0xFB		! mask all irq's but irq2 which
	out	#0x21,al		! is cascaded

	mov	edi, start_linux+0x90000
	call	str_out

; JMP NEW_CS:00100000
	db	0xea
	dd	0x100000; cont1+0x90000;
	dw	0x10			; 0x8 : CE GDT;   0x10: Linux GDT


!
! Delay is needed after doing I/O
!
delay:
	.word	0x00eb			! jmp $+2
	ret

	.org 0x2000

!******************************************************
! convert byte in al to 2 hex digits and store at [edi]
!******************************************************

hex:	mov	bl, al
	shr	al, 4
	add	al, #0x30
	cmp	al, #0x3a
	jb	hex_n1
	add	al, #0x07
hex_n1:	mov	[edi], al
	inc	edi
	mov	al, bl
	and	al, #0x0f
	add	al, #0x30
	cmp	al, #0x3a
	jb	hex_n2
	add	al, #0x07
hex_n2:	mov	[edi], al
	inc	edi
	ret

!******************************************************
! Outputs Character: AL -> COM1
!******************************************************
ch_out:	
!	ret
        push eax

	mov eax, #0x00008000
loop3:	sub eax, #0x1
	jnz loop3
	
ch_w:	mov dx, #0x3f8+0x5
	in al, dx
	and al, 0x40
	jz ch_w

	pop eax
	
ch_outw:	
	mov dx, #0x3f8
	out dx, al			! serial out .... only try
	ret

!******************************************************
! Outputs a String: [EDI] -> COM1
!******************************************************
str_out:
	push	edi
str_rep:
	mov	al, [edi]
	or	al, al
	jz	str_out_end
	call	ch_out
	inc	edi
	jmp	str_rep

str_out_end:
	mov	al, #0x0d
	call	ch_out
	mov	al, #0x0a
	call	ch_out
	pop	edi
	ret


!******************************************************
!**** D A T A *****************************************
!******************************************************

kernel_version:	.ascii "Linux "
		.ascii "                              " ; version goes here
		.byte	0x0

start_linux:	.ascii "Uncompressing Kernel ..."
		.byte	0x0

gdt:
	.word	0,0,0,0		! dummy

	.word	0,0,0,0		! unused

	.word	0xFFFF		! 4Gb - (0x100000*0x1000 = 4Gb)
	.word	0x0000		! base address=0
	.word	0x9A00		! code read/exec
	.word	0x00CF		! granularity=4096, 386 (+5th nibble of limit)

	.word	0xFFFF		! 4Gb - (0x100000*0x1000 = 4Gb)
	.word	0x0000		! base address=0
	.word	0x9200		! data read/write
	.word	0x00CF		! granularity=4096, 386 (+5th nibble of limit)

idt_48:
	.word	0		! idt limit=0
	.long	0		! idt base=0L

gdt_48:
	.word	0x800		! gdt limit=2048, 256 GDT entries
	.long	0x90000+gdt	! gdt base = 0X9xxxx


		.byte	0x0
kernelparam:	.ascii	"console=ttyS0,38400N8"
		.ascii	" root=/dev/hda2 ro "
		.ascii	" reboot=bios"
		.byte	0x0


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

setup_sig1:	.word	0xAA55
setup_sig2:	.word	0x5A5A

